<?php

namespace FORM4\Form4Teaser\Utility;

/**************************************************************
 * Copyright notice
 *
 * (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *
 * All rights reserved
 *
 * This script is part of the TYPO3 project. The TYPO3 project is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 *
 * This script is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * This copyright notice MUST APPEAR in all copies of the script!
 * *************************************************************
 */
use \TYPO3\CMS\Extbase\Persistence\Generic\Query;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;

/**
 * This class provides methods for manipulating the sorting field of a query.
 * Its methods are intended to get registered with \FORM4\Form4Teaser\Utility\ConfigurationUtility::addSorting().
 *
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 * @author Thomas Grothaus <thomas.grothaus@form4.de>
 *
 * @package TYPO3
 * @subpackage form4_teaser
 */
class SortingsUtility {
	
	/**
	 * Sort as in pagetree
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function bySorting(&$settings, Query $query) {
		$settings['sortingField'] = array();
		if ($settings['source'] == 'custom') {
			/*
			 * sorting by custom page list from plugin
			 */
			foreach ($settings['pageUids'] as $pageUid) {
				$settings['sortingField'][] = 'uid=' . $pageUid;
			}
			$settings['sortingField'] = array_reverse($settings['sortingField']);
		} else {
			/*
			 * sorting by pagetree
			 */ 
			$queryGenerator = GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Object\ObjectManager')->get('TYPO3\CMS\Core\Database\QueryGenerator');
			$pageUids = $settings['pageUids'];
			$rootlines = array();
			
			// get rootlines for all pages
			foreach ($pageUids as $pageUid) {
				$rootlines[] = $this->getRootlinePageUids($pageUid);
			}
			
			// get deepest common root page
			$commonRootPageUid = $this->getCommonRootPageUid($rootlines);
			
			// get treelists for the common root page
			// the following code should just fetch the needed part of the treelist - before we had serious performance issues because the whole treelist was always loaded.
			if (is_int($commonRootPageUid)) {
				// if a common root page was found
				$depth = $this->getMaxArraySize($rootlines) - array_search($commonRootPageUid, $rootlines[0]) - 1;
				$treelist = GeneralUtility::intExplode(',', $queryGenerator->getTreeList($commonRootPageUid, $depth, 0, 'true'));
			} else {
				// if no common root page was found, simply use the rootpages of the pages
				$treelist = array();
				$depth = $this->getMaxArraySize($rootlines);
				foreach ($rootlines as $rootline) {
					$treelist = array_merge($treelist, GeneralUtility::intExplode(',', $queryGenerator->getTreeList($rootline[0], $depth, 0, 'true')));
				}
			}
			
			// sort pages by treelist
			usort($pageUids, function ($a, $b) use($treelist) {
				$posA = array_search($a, $treelist);
				$posB = array_search($b, $treelist);
				if ($posA === $posB) {
				} else {
					return ($posA > $posB) ? 1 : -1;
				}
			});
			
			// prepare the orderings for the query 
			foreach ($pageUids as $pageUid) {
				$settings['sortingField'][] = 'uid=' . $pageUid;
			}
			$settings['sortingField'] = array_reverse($settings['sortingField']);
		} 
	}
	
	/**
	 * Returns a array of page Uids of the rootline for a given page
	 * @param integer $pageUid
	 * @return array
	 */
	protected function getRootlinePageUids($pageUid) {
		$pageRepository = GeneralUtility::makeInstance('TYPO3\CMS\Extbase\Object\ObjectManager')->get('TYPO3\CMS\Frontend\Page\PageRepository');
		$uids = array();
		$rootline = $pageRepository->getRootline($pageUid);
		foreach ($rootline as $item) {
			$uids[] = (int)$item['uid'];
		}
		return array_reverse($uids);
	}
	
	/**
	 * Gets the deepest common page UID of the given rootlines.
	 * 
	 * @param array $rootlines Array of arrays with integer elements
	 * @return NULL|integer 
	 */
	protected function getCommonRootPageUid($rootlines) {
		for ($i = 0; $i < sizeof($rootlines); $i++) {
			// return if one of the rootlines is empty
			if (empty($rootlines[$i]) ) {
				return NULL;
			}
			// return if one of the UIDs is different
			if ($i > 0 && $rootlines[$i-1][0] !== $rootlines[$i][0]) {
				return NULL;
			}
		}
		// get page UID and remove the current level from the rootlines
		$commonRootPageUid = $rootlines[0][0];
		foreach ($rootlines as &$rootline) {
			array_shift($rootline);
		}
		
		// get common page for next level
		$nextCommonRootPageUid = $this->getCommonRootPageUid($rootlines);
		if ($nextCommonRootPageUid === NULL) {
			return $commonRootPageUid;
		} else {
			return $nextCommonRootPageUid;
		}
	}
	
	/**
	 * Determines the size of the biggest array.
	 * 
	 * @param array $rootlines Array of arrays
	 * @return integer
	 */
	protected function getMaxArraySize($arrays) {
		$maxSize = 0;
		foreach ($arrays as $array) {
			$size = sizeof($array);
			if ($size > $maxSize) {
				$maxSize = $size;
			}
		}
		return $maxSize;
	}
	
	
	/**
	 * Sort by title
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function byTitle(&$settings, $query) {
		$settings['sortingField'] = 'title';
	}
	
	/**
	 * Sort by the lastUpdated field
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function byLastUpdated(&$settings, $query) {
		$settings['sortingField'] = 'lastUpdated';
	}
	
	/**
	 * Sort by creation date
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function byCrDate(&$settings, $query) {
		$settings['sortingField'] = 'crdate';
	}
	
	/**
	 * Sort by timestamp
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function byTstamp(&$settings, $query) {
		$settings['sortingField'] = 'tstamp';
	}
	
	/**
	 * Sort by starttime
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function byStarttime(&$settings, $query) {
		$settings['sortingField'] = 'starttime';
	}
	
	/**
	 * Sort by endtime
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function byEndtime(&$settings, $query) {
		$settings['sortingField'] = 'endtime';
	}
	
	/**
	 * Sort by the newUntil field
	 *
	 * @param array $settings
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\Query $query
	 * @return void
	 */
	public function byNewUntil(&$settings, $query) {
		$settings['sortingField'] = 'newUntil';
	}
	
}